import 'package:dio/dio.dart';
import 'package:easyflu/config/const.dart';


class HttpError implements Exception {

  late HttpErrorType httpErrorType;

  ///打印给开发者的msg
  late String logMsg;

  ///可自定义，给用户的信息
  late String message;

  late Exception cause;

  HttpError(this.httpErrorType, this.message, this.cause, this.logMsg);


  ///处理dio报错
  HttpError.dioError(DioError error) {
    this.cause = error;
    switch (error.type) {
      case DioErrorType.connectTimeout:
      case DioErrorType.receiveTimeout:
      case DioErrorType.sendTimeout:
        httpErrorType = HttpErrorType.TimeOutError;
        logMsg = '请求超时';
        message = HTTP_TIMEOUT_MSG;
        break;
      case DioErrorType.response:
        httpErrorType = HttpErrorType.ResponseError;
        message = HTTP_ERROR_MSG;
        try {
          int errCode = error.response?.statusCode?? -1;
          switch (errCode) {
            case 400:
              logMsg = '400请求语法错误';
              break;
            case 401:
              logMsg = '401没有权限';
              break;
            case 403:
              logMsg = '403服务器拒绝执行';
              break;
            case 404:
              logMsg = '404无法连接服务器';
              break;
            case 405:
              logMsg = '405请求方法被禁止';
              break;
            case 500:
              logMsg = '500服务器内部错误';
              break;
            case 502:
              logMsg = '502无效的请求';
              break;
            case 503:
              logMsg = '503服务器挂了';
              break;
            case 505:
              logMsg = '505不支持HTTP协议请求';
              break;
            default:
              logMsg = error.response?.statusMessage ?? "";
          }
        } on Exception catch (_) {
          logMsg = '未知错误';
        }
        break;
      case DioErrorType.cancel:
        httpErrorType = HttpErrorType.CancelError;
        logMsg = CANCEL_MSG;
        break;
      case DioErrorType.other:
        httpErrorType = HttpErrorType.Unknown;
        message = UNKNOWN_MSG + ': ' + error.error.toString();
        break;
    }
  }


  ///-------------------------------------------------------HttpError生成---------------------------------------------------
  HttpError.emptyDataError(data){
    httpErrorType = HttpErrorType.EmptyDataError;
    message = NO_DATA_MSG;
    logMsg = 'data数据为空: ${data?.toString()}';
  }

  HttpError.codeError(successCode, code, {String? msg}){
    httpErrorType = HttpErrorType.ResultError;
    message = msg ?? HTTP_INCORRECT_MSG;
    logMsg = '请求code错误，成功code: $successCode, 接口返回code: $code';
  }


  HttpError.emptyMixError(Map<String, HttpError> httpErrorMap, bool syncRequest){
    String errorLogMsg = '';
    httpErrorMap.forEach((url, httpError) {
      errorLogMsg = errorLogMsg + '$url :  ${httpError.message}';
    });
    httpErrorType = HttpErrorType.EmptyDataError;
    message = NO_DATA_MSG;
    logMsg = '${syncRequest? '同步' : '异步'}请求报错: $errorLogMsg';
  }


  HttpError.parseDataError(e, {String? logMsg}){
    httpErrorType = HttpErrorType.DataError;
    message = PARSE_ERROR_MSG;
    logMsg = logMsg??'数据解析错误: $e';
    cause = e is Exception? e : _getUnoKnownException();
  }


  HttpError.unknown(e){
    httpErrorType = HttpErrorType.Unknown;
    message = UNKNOWN_MSG;
    logMsg = '未知错误，请再进一步查看: $e';
    cause = e is Exception? e : _getUnoKnownException();
  }


  HttpError.urlError(String? url){
    httpErrorType = HttpErrorType.UrlError;
    message = UNKNOWN_MSG;
    logMsg = '传入的url不正确：$url';
  }

  Exception _getUnoKnownException(){
    return Exception("未知错误");
  }


  @override
  String toString() {
    return 'HttpError{type: $httpErrorType, errorMsg: $logMsg}';
  }
}


enum HttpErrorType {
  Unknown,
  DataError,
  EmptyDataError,
  NullBodyError,
  CancelError,
  TimeOutError,
  ResponseError,
  ResultError,
  TokenError,
  NetWorkError,
  SSLError,
  UrlError,
}


